@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat.calc;
 
@@ -28,11 +33,12 @@ import org.apfloat.Apint;
 import org.apfloat.ApintMath;
 import org.apfloat.Aprational;
 import org.apfloat.AprationalMath;
+import org.apfloat.spi.Util;
 
 /**
  * Arbitrary precision calculator implementation.
  *
- * @version 1.9.1
+ * @version 1.10.0
  * @author Mikko Tommila
  */
 
@@ -192,6 +198,17 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.cosh((Apcomplex) x);
         }
 
+        @Override
+        public Number euler(Number x)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("Euler can only be used with a valid integer argument");
+            }
+            long n = x.longValue();
+            return ApfloatMath.euler(n);
+        }
+
         @Override
         public Number exp(Number x)
         {
@@ -227,6 +244,18 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.gamma((Apcomplex) x);
         }
 
+        @Override
+        public Number gamma(Number x, Number y)
+        {
+            return ApcomplexMath.gamma((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number gamma(Number x, Number y, Number z)
+        {
+            return ApcomplexMath.gamma((Apcomplex) x, (Apcomplex) y, (Apcomplex) z);
+        }
+
         @Override
         public Number log(Number x)
         {
@@ -251,6 +280,24 @@ public class ApfloatCalculatorImpl
             throw new IllegalArgumentException("Min can only be used with scalar values");
         }
 
+        @Override
+        public Number nextAfter(Number x, Number y)
+        {
+            throw new IllegalArgumentException("Next after can only be used with scalar values");
+        }
+
+        @Override
+        public Number nextDown(Number x)
+        {
+            throw new IllegalArgumentException("Next down can only be used with scalar values");
+        }
+
+        @Override
+        public Number nextUp(Number x)
+        {
+            throw new IllegalArgumentException("Next up can only be used with scalar values");
+        }
+
         @Override
         public Number pi(Number x)
         {
@@ -338,6 +385,12 @@ public class ApfloatCalculatorImpl
             throw new IllegalArgumentException("ToRadians can only be used with scalar values");
         }
 
+        @Override
+        public Number ulp(Number x)
+        {
+            return ApcomplexMath.ulp((Apcomplex) x);
+        }
+
         @Override
         public Number agm(Number x, Number y)
         {
@@ -531,6 +584,24 @@ public class ApfloatCalculatorImpl
             return ApfloatMath.min((Apfloat) x, (Apfloat) y);
         }
 
+        @Override
+        public Number nextAfter(Number x, Number y)
+        {
+            return ApfloatMath.nextAfter((Apfloat) x, (Apfloat) y);
+        }
+
+        @Override
+        public Number nextDown(Number x)
+        {
+            return ApfloatMath.nextDown((Apfloat) x);
+        }
+
+        @Override
+        public Number nextUp(Number x)
+        {
+            return ApfloatMath.nextUp((Apfloat) x);
+        }
+
         @Override
         public Number truncate(Number x)
         {
@@ -746,19 +817,21 @@ public class ApfloatCalculatorImpl
         }
         else if (x instanceof Apfloat)
         {
-            text = ((Apfloat) x).toString(pretty);
+            Apfloat a = (Apfloat) x;
+            a = ApfloatMath.round(a.precision(Util.ifFinite(a.precision(), a.precision() + 1)), a.precision(), RoundingMode.HALF_UP);
+            text = a.toString(pretty);
         }
         else
         {
             Apcomplex z = (Apcomplex) x;
-            String imag = (z.imag().equals(Apfloat.ONE) ? "" : (z.imag().negate().equals(Apfloat.ONE) ? "-" : z.imag().toString(pretty)));
+            String imag = (z.imag().equals(Apfloat.ONE) ? "" : (z.imag().negate().equals(Apfloat.ONE) ? "-" : format(z.imag())));
             if (z.real().signum() == 0)
             {
                 text = imag + "i";
             }
             else
             {
-                text = z.real().toString(pretty) + (z.imag().signum() < 0 ? "" : "+") + imag + "i";
+                text = format(z.real()) + (z.imag().signum() < 0 ? "" : "+") + imag + "i";
             }
         }
         return text;
