@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat.calc;
 
@@ -27,7 +32,7 @@ import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.1
+ * @version 1.10.0
  * @author Mikko Tommila
  */
 
@@ -148,7 +153,7 @@ public class CalculatorTest
         assertCalculation("5e-1", "5e-1");
         assertCalculation("5e-1", "5.e-1");
         assertCalculation("5e1", "5e1");
-        assertCalculation("6.66", "20.0/3.00");
+        assertCalculation("6.67", "20.0/3.00");
         assertCalculation("-1", "I*I");
         assertCalculation("3+2i", "2I+3");
         assertCalculation("3-2i", "i*-2+3");
@@ -199,7 +204,9 @@ public class CalculatorTest
         assertCalculation("9e-1", "frac(2.9)");
         assertCalculation("9/10", "frac(29/10)");
         assertCalculation("6", "gamma(4)");
-        assertCalculation("4.74294943677064514689542753376e1-3.27488916473624576880974867017e1i", "gamma(100.000000000000000000000000000+374.000000000000000000000000000i)");
+        assertCalculation("5.8861", "gamma(4.0000, 1.0000)");
+        assertCalculation("5.0928", "gamma(4.0000, 0, 6.0000)");
+        assertCalculation("4.74294943677064514689542753377e1-3.27488916473624576880974867017e1i", "gamma(100.000000000000000000000000000+374.000000000000000000000000000i)");
         assertCalculation("2", "fmod(5, 3)");
         assertCalculation("5", "hypot(3, 4)");
         assertCalculation("5e-1", "hypot(0.3, 0.4)");
@@ -233,7 +240,7 @@ public class CalculatorTest
         assertCalculation("1.79e2", "toDegrees(3.14)");
         assertCalculation("1.57", "toRadians(90.0)");
         assertCalculation("3.14", "acos(-1.00)");
-        assertCalculation("1", "acosh(1.55)");
+        assertCalculation("1.01", "acosh(1.55)");
         assertCalculation("1.57", "asin(1.00)");
         assertCalculation("0", "asinh(0)");
         assertCalculation("7.85e-1", "atan(1.00)");
@@ -242,28 +249,37 @@ public class CalculatorTest
         assertCalculation("1", "cos(0)");
         assertCalculation("1.54", "cosh(1.00)");
         assertCalculation("8.414709848e-1", "sin(1.000000000)");
-        assertCalculation("1.17", "sinh(1.00)");
-        assertCalculation("1.55", "tan(1.00)");
+        assertCalculation("1.18", "sinh(1.00)");
+        assertCalculation("1.56", "tan(1.00)");
         assertCalculation("9.64e-1", "tanh(2.00)");
+        assertCalculation("5.772e-1", "euler(4)");
         assertCalculation("2.718", "exp(1.000)");
         assertCalculation("1.79+3.14i", "log(-6.00)");
         assertCalculation("1.58", "log(3.00, 2.00)");
-        assertCalculation("1.584", "log(3.000, 2)");
-        assertCalculation("1.5849", "log(3, 2.0000)");
+        assertCalculation("1.585", "log(3.000, 2)");
+        assertCalculation("1.585", "log(3, 2.0000)");
         assertCalculation("1.2", "max(1.1, 1.2)");
         assertCalculation("1/3", "max(1/3, 1/4)");
         assertCalculation("2", "max(1, 2)");
         assertCalculation("1.1", "min(1.1, 1.2)");
         assertCalculation("1/4", "min(1/3, 1/4)");
         assertCalculation("1", "min(1, 2)");
+        assertCalculation("2", "nextAfter(1., 2)");
+        assertCalculation("2", "nextUp(1.)");
+        assertCalculation("1", "nextDown(2.)");
+        assertCalculation("2/3", "nextDown(2/3)");
+        assertCalculation("1", "ulp(2.)");
+        assertCalculation("0", "ulp(2/3)");
+        assertCalculation("0", "ulp(1+i)");
+        assertCalculation("1e-1", "ulp(-10.00-2.0i)");
         assertCalculation("1.57", "arg(1.00i)");
         assertCalculation("2-i", "conj(2+i)");
         assertCalculation("1", "imag(2+i)");
         assertCalculation("2", "real(2+i)");
         assertCalculation("2.47468", "agm(2.000000, 3.000000)");
-        assertCalculation("5.671e-1", "w(1.000)");
+        assertCalculation("5.672e-1", "w(1.000)");
         assertCalculation("-3.181e-1+1.3372i", "w(-1.0000)");
-        assertCalculation("-1.34284+5.24724i", "w(1.00000+i, 1)");
+        assertCalculation("-1.34285+5.24725i", "w(1.00000+i, 1)");
         assertCalculation("3", "gcd(15, 12)");
         assertCalculation("60", "lcm(15, 12)");
         assertCalculation("3.14159", "pi(6)");
@@ -299,7 +315,7 @@ public class CalculatorTest
         assertCalculationFailure("frac(2, 2)");
         assertCalculationFailure("gamma(0)");
         assertCalculationFailure("gamma()");
-        assertCalculationFailure("gamma(2, 2)");
+        assertCalculationFailure("gamma(2, 2, 2, 2)");
         assertCalculationFailure("fmod(2, i)");
         assertCalculationFailure("fmod(2)");
         assertCalculationFailure("fmod(2, 2, 2)");
@@ -375,6 +391,10 @@ public class CalculatorTest
         assertCalculationFailure("tan(1, 1)");
         assertCalculationFailure("tanh()");
         assertCalculationFailure("tanh(1, 1)");
+        assertCalculationFailure("euler()");
+        assertCalculationFailure("euler(0.5)");
+        assertCalculationFailure("euler(i)");
+        assertCalculationFailure("euler(1, 1)");
         assertCalculationFailure("exp()");
         assertCalculationFailure("exp(1, 1)");
         assertCalculationFailure("log()");
@@ -385,6 +405,17 @@ public class CalculatorTest
         assertCalculationFailure("min()");
         assertCalculationFailure("min(1)");
         assertCalculationFailure("min(1, 1, 1)");
+        assertCalculationFailure("nextAfter(1)");
+        assertCalculationFailure("nextAfter(1, 2, 3)");
+        assertCalculationFailure("nextAfter(1, i)");
+        assertCalculationFailure("nextUp()");
+        assertCalculationFailure("nextUp(1, 1)");
+        assertCalculationFailure("nextUp(i)");
+        assertCalculationFailure("nextDown()");
+        assertCalculationFailure("nextDown(2, 2)");
+        assertCalculationFailure("nextDown(i)");
+        assertCalculationFailure("ulp()");
+        assertCalculationFailure("ulp(2, 2)");
         assertCalculationFailure("arg()");
         assertCalculationFailure("arg(1, 1)");
         assertCalculationFailure("conj()");
