@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -38,7 +43,7 @@ import static java.util.FormattableFlags.*;
  * @see Apfloat
  * @see ApcomplexMath
  *
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -111,7 +116,7 @@ public class Apcomplex
 
     public Apcomplex(Apfloat real)
     {
-        this(real, ZERO);
+        this(real, ZEROS[real.radix()]);
     }
 
     /**
@@ -155,7 +160,7 @@ public class Apcomplex
         if (!value.startsWith("("))
         {
             this.real = new Apfloat(value);
-            this.imag = ZERO;
+            this.imag = ZEROS[this.real.radix()];
             return;
         }
         if (!value.endsWith(")"))
@@ -167,7 +172,7 @@ public class Apcomplex
         if (index < 0)
         {
             this.real = new Apfloat(value.substring(1, value.length() - 1).trim());
-            this.imag = ZERO;
+            this.imag = ZEROS[this.real.radix()];
             return;
         }
 
@@ -199,7 +204,7 @@ public class Apcomplex
         if (!ApfloatHelper.readMatch(in, '('))
         {
             this.real = new Apfloat(in);
-            this.imag = ZERO;
+            this.imag = ZEROS[this.real.radix()];
             return;
         }
 
@@ -214,7 +219,7 @@ public class Apcomplex
         }
         else
         {
-            this.imag = ZERO;
+            this.imag = ZEROS[this.real.radix()];
         }
 
         ApfloatHelper.extractWhitespace(in);
@@ -335,8 +340,8 @@ public class Apcomplex
         long realPrecision = precisions[0],
              imagPrecision = precisions[1];
 
-        return new Apcomplex(realPrecision > 0 ? z.real().precision(realPrecision) : Apfloat.ZERO,
-                             imagPrecision > 0 ? z.imag().precision(imagPrecision) : Apfloat.ZERO);
+        return new Apcomplex(realPrecision > 0 ? z.real().precision(realPrecision) : ZEROS[real().radix()],
+                             imagPrecision > 0 ? z.imag().precision(imagPrecision) : ZEROS[imag().radix()]);
     }
 
     /**
@@ -801,7 +806,8 @@ public class Apcomplex
         else if (obj instanceof Apcomplex)
         {
             Apcomplex that = (Apcomplex) obj;
-            return real().equals(that.real()) &&
+            return test(that) &&
+                   real().equals(that.real()) &&
                    imag().equals(that.imag());
         }
         else
@@ -810,6 +816,26 @@ public class Apcomplex
         }
     }
 
+    /**
+     * Tests two apcomplex numbers for equality.
+     * Returns <code>false</code> if the numbers are definitely known to be not equal.
+     * If <code>true</code> is returned, equality is unknown and should be verified by
+     * calling {@link #equals(Object)}.
+     * This method is usually significantly faster than calling <code>equals(Object)</code>.
+     *
+     * @param z The number to test against.
+     *
+     * @return <code>false</code> if the numbers are definitely not equal, <code>true</code> if unknown.
+     *
+     * @since 1.10.0
+     */
+
+    public boolean test(Apcomplex z)
+        throws ApfloatRuntimeException
+    {
+        return real().test(z.real()) && imag().test(z.imag());
+    }
+
     /**
      * Returns a hash code for this apcomplex.
      *
@@ -949,15 +975,19 @@ public class Apcomplex
         }
     }
 
+    static final Apint[] ZEROS;
     static final Apint[] ONES;
 
     static
     {
+        ZEROS = new Apint[37];
         ONES = new Apint[37];
         for (int i =  2; i <= 36; i++)
         {
+            ZEROS[i] = new Apint(0, i);
             ONES[i] = new Apint(1, i);
         }
+        ZEROS[ZERO.radix()] = ZERO;
         ONES[ONE.radix()] = ONE;
     }
 
