@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -30,7 +35,7 @@ import org.apfloat.spi.Util;
  *
  * @see ApfloatMath
  *
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -153,7 +158,7 @@ public class ApcomplexMath
         }
         else if (n < 0)
         {
-            z = Apcomplex.ONE.divide(z);
+            z = Apcomplex.ONES[z.radix()].divide(z);
             n = -n;
         }
 
@@ -269,7 +274,7 @@ public class ApcomplexMath
             {
                 throw new ArithmeticException("Inverse root of zero");
             }
-            return Apcomplex.ZERO;              // Avoid division by zero
+            return Apcomplex.ZEROS[z.radix()];  // Avoid division by zero
         }
         else if (n == 1)
         {
@@ -377,7 +382,7 @@ public class ApcomplexMath
         {
             // Avoid round-off errors and produce a pure imaginary result
             Apfloat y = ApfloatMath.inverseRoot(z.real().negate(), n);
-            return new Apcomplex(Apfloat.ZERO, k == 0 ? y.negate() : y);
+            return new Apcomplex(Apfloat.ZEROS[z.radix()], k == 0 ? y.negate() : y);
         }
 
         long targetPrecision = z.precision();
@@ -613,7 +618,7 @@ public class ApcomplexMath
                 throw new ArithmeticException("Inverse root of zero");
             }
             Apcomplex[] allRoots = new Apcomplex[n];
-            Arrays.fill(allRoots, Apcomplex.ZERO);
+            Arrays.fill(allRoots, Apcomplex.ZEROS[z.radix()]);
             return allRoots;                                    // Avoid division by zero
         }
 
@@ -652,11 +657,28 @@ public class ApcomplexMath
         if (a.real().signum() == 0 && a.imag().signum() == 0 ||
             b.real().signum() == 0 && b.imag().signum() == 0)         // Would not converge quadratically
         {
-            return Apcomplex.ZERO;
+            return Apcomplex.ZEROS[a.radix()];
+        }
+
+        if (a.real().signum() == b.real().signum() &&
+            a.imag().signum() == 0 &&
+            b.imag().signum() == 0)
+        {
+            return ApfloatMath.agm(a.real(), b.real());
+        }
+
+        if (a.equals(b))                                              // Thanks to Marko Gaspersic for finding several bugs in issue #12
+        {
+            return a.precision(Math.min(a.precision(), b.precision()));
+        }
+
+        if (a.equals(b.negate()))                                     // Would not converge quadratically
+        {
+            return Apcomplex.ZEROS[a.radix()];
         }
 
         long workingPrecision = Math.min(a.precision(), b.precision()),
-             targetPrecision = Math.max(a.precision(), b.precision());
+             targetPrecision = workingPrecision;
 
         if (workingPrecision == Apfloat.INFINITE)
         {
@@ -677,7 +699,7 @@ public class ApcomplexMath
         while (precision < CONVERGING && precision < halfWorkingPrecision)
         {
             Apcomplex t = a.add(b).divide(two);
-            b = sqrt(a.multiply(b));
+            b = rightSqrt(a.multiply(b), t);
             a = t;
 
             // Conserve precision in case of accumulating round-off errors
@@ -691,7 +713,7 @@ public class ApcomplexMath
         while (precision <= halfWorkingPrecision)
         {
             Apcomplex t = a.add(b).divide(two);
-            b = sqrt(a.multiply(b));
+            b = rightSqrt(a.multiply(b), t);
             a = t;
 
             // Conserve precision in case of accumulating round-off errors
@@ -704,6 +726,31 @@ public class ApcomplexMath
         return ApfloatHelper.setPrecision(a.add(b).divide(two), targetPrecision);
     }
 
+    private static Apcomplex rightSqrt(Apcomplex z, Apcomplex reference)
+    {
+        // See  D. A. Cox, "The Arithmetic-Geometric Mean of Gauss", L'Enseignement Mathematique, Vol. 30, 1984, pp. 275-330
+        // or for example Tomack Gilmore's paper about it: https://homepage.univie.ac.at/tomack.gilmore/papers/Agm.pdf
+        // 1. norm(a1 - b1) <= norm(a1 + b1)
+        // 2. If norm(a1 - b1) = norm(a1 + b1) then imag(b1 / a1) > 0
+        Apcomplex result = sqrt(z);
+
+        // First compare norms with low precision
+        int doublePrecision = ApfloatHelper.getDoublePrecision(z.radix());
+        Apcomplex approxResult = result.precision(doublePrecision);
+        Apcomplex approxReference = reference.precision(doublePrecision);
+        int comparison = norm(approxReference.subtract(approxResult)).compareTo(norm(approxReference.add(approxResult)));
+        if (comparison == 0)
+        {
+            // Full precision comparison as they are equal to low precision
+            comparison = norm(reference.subtract(result)).compareTo(norm(reference.add(result)));
+        }
+        if (comparison > 0 || comparison == 0 && result.divide(reference).imag().signum() <= 0)
+        {
+            result = result.negate();
+        }
+        return result;
+    }
+
     /**
      * Natural logarithm.<p>
      *
@@ -924,7 +971,7 @@ public class ApcomplexMath
         {
             // Underflow
 
-            return Apcomplex.ZERO;
+            return Apcomplex.ZEROS[z.radix()];
         }
 
         boolean negateResult = false;                           // If the final result is to be negated
@@ -1000,7 +1047,7 @@ public class ApcomplexMath
 
             if (resultReal.signum() == 0) {
                 // Underflow
-                return Apcomplex.ZERO;
+                return Apcomplex.ZEROS[z.radix()];
             }
         }
 
@@ -1152,7 +1199,11 @@ public class ApcomplexMath
         Apcomplex i = new Apcomplex(Apfloat.ZERO, one),
                   w = i.multiply(log(z.add(sqrt(z.multiply(z).subtract(one)))));
 
-        if (z.real().signum() * z.imag().signum() >= 0)
+        if (z.real().signum() < 0 && z.imag().signum() == 0)
+        {
+            return new Apcomplex(w.real().negate(), w.imag());
+        }
+        else if (z.real().signum() * z.imag().signum() > 0 || z.real().signum() == 0)
         {
             return w.negate();
         }
@@ -1175,7 +1226,7 @@ public class ApcomplexMath
     {
         Apfloat one = new Apfloat(1, Apfloat.INFINITE, z.radix());
 
-        if (z.real().signum() >= 0)
+        if (z.real().signum() > 0 || z.real().signum() == 0 && z.imag().signum() >= 0)
         {
             return log(z.add(sqrt(z.multiply(z).subtract(one))));
         }
@@ -1258,9 +1309,17 @@ public class ApcomplexMath
 
         Apfloat one = new Apfloat(1, Apfloat.INFINITE, z.radix()),
                 two = new Apfloat(2, Apfloat.INFINITE, z.radix());
-        Apcomplex i = new Apcomplex(Apfloat.ZERO, one);
+        Apcomplex i = new Apcomplex(Apfloat.ZERO, one),
+                  w = log(i.add(z).divide(i.subtract(z))).multiply(i).divide(two);
 
-        return log(i.add(z).divide(i.subtract(z))).multiply(i).divide(two);
+        if (z.real().signum() == 0 && z.imag().signum() > 0)
+        {
+            return new Apcomplex(w.real().negate(), w.imag());
+        }
+        else
+        {
+            return w;
+        }
     }
 
     /**
@@ -1278,8 +1337,16 @@ public class ApcomplexMath
     {
         Apfloat one = new Apfloat(1, Apfloat.INFINITE, z.radix()),
                 two = new Apfloat(2, Apfloat.INFINITE, z.radix());
+        Apcomplex w = log(one.add(z).divide(one.subtract(z))).divide(two);
 
-        return log(one.add(z).divide(one.subtract(z))).divide(two);
+        if (z.real().signum() > 0 && z.imag().signum() == 0)
+        {
+            return w.conj();
+        }
+        else
+        {
+            return w;
+        }
     }
 
     /**
@@ -1518,7 +1585,7 @@ public class ApcomplexMath
         {
             if (z[i].real().signum() == 0 && z[i].imag().signum() == 0)
             {
-                return Apcomplex.ZERO;
+                return Apcomplex.ZEROS[z[i].radix()];
             }
             maxPrec = Math.min(maxPrec, z[i].precision());
         }
@@ -1692,6 +1759,77 @@ public class ApcomplexMath
         return result.precision(precision);
     }
 
+    /**
+     * Incomplete gamma function.<p>
+     *
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the incomplete gamma function.
+     *
+     * @param a The first argument.
+     * @param z The second argument.
+     *
+     * @return <code>&Gamma;(a, z)</code>
+     *
+     * @throws ArithmeticException If the real part of <code>a</code> is nonpositive and <code>z</code> is zero.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apcomplex gamma(Apcomplex a, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return IncompleteGammaHelper.gamma(a, z);
+    }
+
+    /**
+     * Generalized incomplete gamma function.<p>
+     *
+     * This function is defined as: <code>&Gamma;(a, z0, z1) = &Gamma;(a, z0) - &Gamma;(a, z1)</code><p>
+     *
+     * The lower gamma function can be calculated with: <code>&gamma;(a, z) = &Gamma;(a, 0, z)</code><p>
+     *
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the incomplete gamma function.
+     *
+     * @param a The first argument.
+     * @param z0 The second argument.
+     * @param z1 The third argument.
+     *
+     * @return <code>&Gamma;(a, z0, z1)</code>
+     *
+     * @throws ArithmeticException If the real part of <code>a</code> is nonpositive and either <code>z0</code> or <code>z1</code> is zero. For the lower gamma function if <code>a</code> is a nonpositive integer.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apcomplex gamma(Apcomplex a, Apcomplex z0, Apcomplex z1)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return IncompleteGammaHelper.gamma(a, z0, z1);
+    }
+
+    /**
+     * Returns the unit in the last place of the argument, considering the
+     * scale and precision. This is maximum of the ulps of the real and
+     * imaginary part of the argument.
+     * If the precision of the argument is infinite, zero is returned.
+     *
+     * @param z The argument.
+     *
+     * @return The ulp of the argument.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat ulp(Apcomplex z)
+    {
+        return ApfloatMath.max(ApfloatMath.ulp(z.real()), ApfloatMath.ulp(z.imag()));
+    }
+
     // Extend the precision on last iteration
     private static Apcomplex lastIterationExtendPrecision(int iterations, int precisingIteration, Apcomplex z)
     {
