@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -35,7 +40,7 @@ import static org.apfloat.spi.RadixConstants.*;
 /**
  * Various utility methods related to apfloats.
  *
- * @version 1.6.2
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -501,6 +506,25 @@ class ApfloatHelper
         return extendPrecision(precision, Apfloat.EXTRA_PRECISION);
     }
 
+    // Returns given precision reduced by specified amount
+    public static long reducePrecision(long precision, long extraPrecision)
+        throws ApfloatRuntimeException
+    {
+        precision = precision - extraPrecision;
+        if (precision <= 0)
+        {
+            throw new LossOfPrecisionException("Complete loss of precision");
+        }
+        return precision;
+    }
+
+    // Returns given precision reduced by Apfloat.EXTRA_PRECISION
+    public static long reducePrecision(long precision)
+        throws ApfloatRuntimeException
+    {
+        return reducePrecision(precision, Apfloat.EXTRA_PRECISION);
+    }
+
     // Returns x with precision extended by Apfloat.EXTRA_PRECISION
     public static Apfloat extendPrecision(Apfloat x)
         throws ApfloatRuntimeException
@@ -515,6 +539,20 @@ class ApfloatHelper
         return x.precision(extendPrecision(x.precision(), extraPrecision));
     }
 
+    // Returns x with precision reduced by Apfloat.EXTRA_PRECISION
+    public static Apfloat reducePrecision(Apfloat x)
+        throws ApfloatRuntimeException
+    {
+        return x.precision(reducePrecision(x.precision()));
+    }
+
+    // Returns x with precision reduced by specified amount
+    public static Apfloat reducePrecision(Apfloat x, long extraPrecision)
+        throws ApfloatRuntimeException
+    {
+        return x.precision(reducePrecision(x.precision(), extraPrecision));
+    }
+
     // Returns z with precision as specified
     public static Apcomplex setPrecision(Apcomplex z, long precision)
         throws ApfloatRuntimeException
@@ -540,13 +578,13 @@ class ApfloatHelper
         {
             if (realPrecision + precisionChange <= 0)
             {
-                return new Apcomplex(Apfloat.ZERO,
+                return new Apcomplex(Apfloat.ZEROS[z.radix()],
                                      z.imag().precision(precision));
             }
             else if (imagPrecision + precisionChange <= 0)
             {
                 return new Apcomplex(z.real().precision(precision),
-                                     Apfloat.ZERO);
+                                     Apfloat.ZEROS[z.radix()]);
             }
         }
 
@@ -558,32 +596,48 @@ class ApfloatHelper
     public static Apcomplex limitPrecision(Apcomplex z, long precision)
         throws ApfloatRuntimeException
     {
-        return new Apcomplex(z.real().precision(Math.min(z.real().precision(), precision)),
-                             z.imag().precision(Math.min(z.imag().precision(), precision)));
+        return new Apcomplex(limitPrecision(z.real(), precision),
+                             limitPrecision(z.imag(), precision));
     }
 
     // Returns z with precision at least as specified
     public static Apcomplex ensurePrecision(Apcomplex z, long precision)
         throws ApfloatRuntimeException
     {
-        return new Apcomplex(z.real().precision(Math.max(z.real().precision(), precision)),
-                             z.imag().precision(Math.max(z.imag().precision(), precision)));
+        return new Apcomplex(ensurePrecision(z.real(), precision),
+                             ensurePrecision(z.imag(), precision));
     }
 
     // Returns z with precision extended by Apfloat.EXTRA_PRECISION
     public static Apcomplex extendPrecision(Apcomplex z)
         throws ApfloatRuntimeException
     {
-        return new Apcomplex(z.real().precision(extendPrecision(z.real().precision())),
-                             z.imag().precision(extendPrecision(z.imag().precision())));
+        return new Apcomplex(extendPrecision(z.real()),
+                             extendPrecision(z.imag()));
     }
 
     // Returns z with precision extended by specified precision
     public static Apcomplex extendPrecision(Apcomplex z, long extraPrecision)
         throws ApfloatRuntimeException
     {
-        return new Apcomplex(z.real().precision(extendPrecision(z.real().precision(), extraPrecision)),
-                             z.imag().precision(extendPrecision(z.imag().precision(), extraPrecision)));
+        return new Apcomplex(extendPrecision(z.real(), extraPrecision),
+                             extendPrecision(z.imag(), extraPrecision));
+    }
+
+    // Returns z with precision reduced by Apfloat.EXTRA_PRECISION
+    public static Apcomplex reducePrecision(Apcomplex z)
+        throws ApfloatRuntimeException
+    {
+        return new Apcomplex(reducePrecision(z.real()),
+                             reducePrecision(z.imag()));
+    }
+
+    // Returns z with precision reduced by specified amount
+    public static Apcomplex reducePrecision(Apcomplex z, long extraPrecision)
+        throws ApfloatRuntimeException
+    {
+        return new Apcomplex(reducePrecision(z.real(), extraPrecision),
+                             reducePrecision(z.imag(), extraPrecision));
     }
 
     public static long size(Aprational x)
