@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -42,7 +47,7 @@ import org.apfloat.spi.Util;
  *
  * @see ApintMath
  *
- * @version 1.9.1
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -162,7 +167,7 @@ public class ApfloatMath
         }
         else if (x.signum() == 0)
         {
-            return Apfloat.ZERO;                // Avoid division by zero
+            return Apfloat.ZEROS[x.radix()];    // Avoid division by zero
         }
         else if (n == 1)
         {
@@ -638,7 +643,7 @@ public class ApfloatMath
         }
         else if (x.precision() <= x.scale() - y.scale())                        // We now know that x.scale() >= y.scale()
         {
-            return Apfloat.ZERO;                // Degenerate case; not enough precision to make any sense
+            return Apfloat.ZEROS[x.radix()];    // Degenerate case; not enough precision to make any sense
         }
         else
         {
@@ -721,7 +726,7 @@ public class ApfloatMath
         precisions = ApfloatHelper.getMatchingPrecisions(a, b, c, d);
         if (precisions[0] == 0)
         {
-            ab = Apfloat.ZERO;
+            ab = Apfloat.ZEROS[a.radix()];
         }
         else
         {
@@ -731,7 +736,7 @@ public class ApfloatMath
         }
         if (precisions[1] == 0)
         {
-            cd = Apfloat.ZERO;
+            cd = Apfloat.ZEROS[c.radix()];
         }
         else
         {
@@ -759,11 +764,28 @@ public class ApfloatMath
     {
         if (a.signum() == 0 || b.signum() == 0)         // Would not converge quadratically
         {
-            return Apfloat.ZERO;
+            return Apfloat.ZEROS[a.radix()];
+        }
+
+        if (abs(a).equals(abs(b)))                      // Would not converge quadratically
+        {
+            return a.signum() == b.signum() ? a.precision(Math.min(a.precision(), b.precision())) : Apfloat.ZEROS[a.radix()];
+        }
+
+        if (a.signum() != b.signum())
+        {
+            throw new ArithmeticException("Non-real result");
+        }
+
+        boolean negate = a.signum() < 0;                // Thanks to Marko Gaspersic for finding several bugs in issue #12
+        if (negate)
+        {
+            a = a.negate();
+            b = b.negate();
         }
 
         long workingPrecision = Math.min(a.precision(), b.precision()),
-             targetPrecision = Math.max(a.precision(), b.precision());
+             targetPrecision = workingPrecision;
 
         if (workingPrecision == Apfloat.INFINITE)
         {
@@ -808,7 +830,8 @@ public class ApfloatMath
             precision *= 2;
         }
 
-        return a.add(b).divide(two).precision(targetPrecision);
+        Apfloat result = a.add(b).divide(two).precision(targetPrecision);
+        return (negate ? result.negate() : result);
     }
 
     /**
@@ -1156,7 +1179,7 @@ public class ApfloatMath
         }
         else if (x.equals(Apfloat.ONE))
         {
-            return Apfloat.ZERO;
+            return Apfloat.ZEROS[x.radix()];
         }
 
         // Calculate the log using 1 / radix <= x < 1 and the log addition formula
@@ -1343,7 +1366,7 @@ public class ApfloatMath
         {
             // Underflow
 
-            return Apfloat.ZERO;
+            return Apfloat.ZEROS[radix];
         }
         else if (x.scale() <= Long.MIN_VALUE / 2 + Apfloat.EXTRA_PRECISION)
         {
@@ -1375,9 +1398,10 @@ public class ApfloatMath
             result = new Apfloat(Math.pow((double) radix, fractionalPart.doubleValue()), doublePrecision, radix);
             result = scale(result, integerPart.longValue());
 
-            if (result.signum() == 0) {
+            if (result.signum() == 0)
+            {
                 // Underflow
-                return Apfloat.ZERO;
+                return Apfloat.ZEROS[radix];
             }
 
             precision = doublePrecision;
@@ -1703,7 +1727,7 @@ public class ApfloatMath
         {
             if (x.signum() > 0)
             {
-                return Apfloat.ZERO;
+                return Apfloat.ZEROS[x.radix()];
             }
 
             return pi(x.precision(), x.radix());
@@ -1809,7 +1833,7 @@ public class ApfloatMath
     public static Apfloat toDegrees(Apfloat x)
         throws ApfloatRuntimeException
     {
-        return x.multiply(new Apfloat(180, Apfloat.INFINITE, x.radix())).divide(pi(x.precision(), x.radix()));
+        return x.signum() == 0 ? x : x.multiply(new Apfloat(180, Apfloat.INFINITE, x.radix())).divide(pi(x.precision(), x.radix()));    // Thanks to Marko Gaspersic for finding the bug in issue #11
     }
 
     /**
@@ -1825,7 +1849,7 @@ public class ApfloatMath
     public static Apfloat toRadians(Apfloat x)
         throws ApfloatRuntimeException
     {
-        return x.divide(new Apfloat(180, Apfloat.INFINITE, x.radix())).multiply(pi(x.precision(), x.radix()));
+        return x.signum() == 0 ? x : x.divide(new Apfloat(180, Apfloat.INFINITE, x.radix())).multiply(pi(x.precision(), x.radix()));    // Thanks to Marko Gaspersic for finding the bug in issue #11
     }
 
     /**
@@ -1858,7 +1882,7 @@ public class ApfloatMath
         {
             if (x[i].signum() == 0)
             {
-                return Apfloat.ZERO;
+                return Apfloat.ZEROS[x[i].radix()];
             }
             maxPrec = Math.min(maxPrec, x[i].precision());
         }
@@ -2015,6 +2039,76 @@ public class ApfloatMath
         return s;
     }
 
+    /**
+     * Calculates &gamma;, the Euler-Mascheroni constant. Uses default radix.
+     *
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     *
+     * @param precision Number of digits of &gamma; to calculate.
+     *
+     * @return &gamma; accurate to <code>precision</code> digits, in the default radix.
+     *
+     * @exception java.lang.NumberFormatException If the default radix is not valid.
+     * @exception java.lang.IllegalArgumentException In case the precision is invalid.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat euler(long precision)
+    {
+        ApfloatContext ctx = ApfloatContext.getContext();
+        int radix = ctx.getDefaultRadix();
+
+        return euler(precision, radix);
+    }
+
+    /**
+     * Calculates &gamma;, the Euler-Mascheroni constant.<p>
+     *
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     *
+     * @param precision Number of digits of &gamma; to calculate.
+     * @param radix The radix in which the number should be presented.
+     *
+     * @return &gamma; accurate to <code>precision</code> digits, in base <code>radix</code>.
+     *
+     * @exception java.lang.NumberFormatException If the radix is not valid.
+     * @exception java.lang.IllegalArgumentException In case the precision is invalid.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat euler(long precision, int radix)
+        throws IllegalArgumentException, NumberFormatException, ApfloatRuntimeException
+    {
+        // See https://www.ams.org/journals/mcom/1980-34-149/S0025-5718-1980-0551307-4/S0025-5718-1980-0551307-4.pdf
+        // Mathematics of Computation, volume 34, number 149, January 1980, pages 305-312
+        // "Some new algorithms for high-precision computation of Euler's constant" by Richard P. Brent and Edwin M. McMillan
+        // Note that a faster algorithm using binary splitting would be possible but isn't implemented here. After all this is mostly just needed with the gamma functions, which are slow anyways.
+        long n = (long) (Apfloat.EXTRA_PRECISION + 0.25 * Math.log(radix) * precision),
+             workingPrecision = ApfloatHelper.extendPrecision(precision);
+        Apfloat a = log(new Apfloat(n, workingPrecision, radix)).negate(),
+                b = new Apfloat(1, workingPrecision, radix),
+                u = a,
+                v = b,
+                ou,
+                ov;
+        Apint one = new Apint(1, radix),
+              k = one,
+              n2 = ApintMath.pow(new Apint(n, radix), 2);
+        do
+        {
+            ou = u;
+            ov = v;
+            b = b.multiply(n2).divide(pow(k, 2));
+            a = a.multiply(n2).divide(k).add(b).divide(k);
+            u = u.add(a);
+            v = v.add(b);
+            k = k.add(one);
+        } while (u.equalDigits(ou) < precision || v.equalDigits(ov) < precision);
+        return u.divide(v).precision(precision);
+    }
+
     /**
      * Gamma function.<p>
      *
@@ -2037,6 +2131,67 @@ public class ApfloatMath
         return ApcomplexMath.gamma(x).real();
     }
 
+    /**
+     * Incomplete gamma function.<p>
+     *
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the gamma function.
+     *
+     * @param a The first argument.
+     * @param x The second argument.
+     *
+     * @return <code>&Gamma;(a, x)</code>
+     *
+     * @throws ArithmeticException If <code>a</code> is not a positive integer and <code>x</code> is nonpositive.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat gamma(Apfloat a, Apfloat x)
+    {
+        if (x.signum() < 0 && !(a.signum() > 0 && a.isInteger()))
+        {
+            // For a real result, in the case x < 0, a must be a positive integer
+            throw new ArithmeticException("Non-real result");
+        }
+        return ApcomplexMath.gamma(a, x).real();
+    }
+
+    /**
+     * Generalized incomplete gamma function.<p>
+     *
+     * This function is defined as: <code>&Gamma;(a, x0, x1) = &Gamma;(a, x0) - &Gamma;(a, x1)</code><p>
+     *
+     * The lower gamma function can be calculated with: <code>&gamma;(a, x) = &Gamma;(a, 0, x)</code><p>
+     *
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is something like O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the gamma function.
+     *
+     * @param a The first argument.
+     * @param x0 The second argument.
+     * @param x1 The third argument.
+     *
+     * @return <code>&Gamma;(a, x0, x1)</code>
+     *
+     * @throws ArithmeticException If <code>a</code> is not a positive integer and either <code>x0</code> or <code>x1</code> is nonpositive.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat gamma(Apfloat a, Apfloat x0, Apfloat x1)
+    {
+        if ((x0.signum() < 0 || x1.signum() < 0) && !(a.signum() > 0 && a.isInteger()))
+        {
+            // For a real result, in the case x < 0, a must be a positive integer
+            throw new ArithmeticException("Non-real result");
+        }
+        return ApcomplexMath.gamma(a, x0, x1).real();
+    }
+
     /**
      * Generates a random number. Uses the default radix.
      * Returned values are chosen pseudorandomly with (approximately)
@@ -2198,6 +2353,98 @@ public class ApfloatMath
         return (x.compareTo(y) < 0 ? x : y);
     }
 
+    /**
+     * Returns the number adjacent to the first argument in the direction of
+     * the second argument, considering the scale and precision of the first
+     * argument. If the precision of the first argument is infinite, the
+     * first argument is returned. If both arguments compare as equal then
+     * the first argument is returned.
+     *
+     * @param start The starting value.
+     * @param direction Value indicating which of <code>start</code>'s neighbors or <code>start</code> should be returned.
+     *
+     * @return The number adjacent to <code>start</code> in the direction of <code>direction</code>.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat nextAfter(Apfloat start, Apfloat direction)
+    {
+        return nextInDirection(start, direction.compareTo(start));
+    }
+
+    /**
+     * Returns the number adjacent to the argument in the direction of
+     * positive infinity, considering the scale and precision of the
+     * argument. If the precision of the argument is infinite, the
+     * argument is returned.
+     *
+     * @param x The starting value.
+     *
+     * @return The adjacent value closer to positive infinity.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat nextUp(Apfloat x)
+    {
+        return nextInDirection(x, 1);
+    }
+
+    /**
+     * Returns the number adjacent to the argument in the direction of
+     * negative infinity, considering the scale and precision of the
+     * argument. If the precision of the argument is infinite, the
+     * argument is returned.
+     *
+     * @param x The starting value.
+     *
+     * @return The adjacent value closer to negative infinity.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat nextDown(Apfloat x)
+    {
+        return nextInDirection(x, -1);
+    }
+
+    /**
+     * Returns the unit in the last place of the argument, considering the
+     * scale and precision. This is same as the difference between the argument
+     * and the value returned from {@link #nextUp(Apfloat)}.
+     * If the precision of the argument is infinite, zero is returned.<p>
+     *
+     * For example, ulp of <code>1.</code> is <code>1</code>, ulp of <code>1.1</code> is <code>0.1</code>
+     * and ulp of <code>1.001</code> is <code>0.001</code> (considering significant digits only).
+     *
+     * @param x The argument.
+     *
+     * @return The ulp of the argument.
+     *
+     * @since 1.10.0
+     */
+
+    public static Apfloat ulp(Apfloat x)
+    {
+        return ulp(x, 1);
+    }
+
+    private static Apfloat nextInDirection(Apfloat x, int direction)
+    {
+        return x.add(ulp(x, direction));
+    }
+
+    private static Apfloat ulp(Apfloat x, int direction)
+    {
+        long scale = x.scale() - x.precision();
+        if (x.precision() == Apfloat.INFINITE || x.scale() < 0 && scale >= 0)   // Detect overflow
+        {
+            return Apfloat.ZEROS[x.radix()];
+        }
+        return scale(new Apfloat(direction, 1, x.radix()), scale);
+    }
+
     // Extend the precision on last iteration
     private static Apfloat lastIterationExtendPrecision(int iterations, int precisingIteration, Apfloat x)
         throws ApfloatRuntimeException
