@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -26,7 +31,7 @@ import java.util.Random;
 /**
  * Various mathematical functions for arbitrary precision integers.
  *
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -61,7 +66,7 @@ public class ApintMath
         }
         else if (n < 0)
         {
-            return Apint.ZERO;
+            return Apint.ZEROS[x.radix()];
         }
 
         // Algorithm improvements by Bernd Kellner
@@ -155,11 +160,11 @@ public class ApintMath
         }
         else if (x.equals(Apint.ONE) || n == 1)
         {
-            return new Apint[] { x, Apint.ZERO };
+            return new Apint[] { x, Apint.ZEROS[x.radix()] };
         }
         else if (n < 0)
         {
-            return new Apint[] { Apint.ZERO, x };               // 1 / x where x > 1
+            return new Apint[] { Apint.ZEROS[x.radix()], x };   // 1 / x where x > 1
         }
 
         long precision = x.scale() / n + Apint.EXTRA_PRECISION;
@@ -354,7 +359,7 @@ public class ApintMath
         else if (y.equals(Apint.ONE))
         {
             // x / 1 = x
-            return new Apint[] { x, Apint.ZERO };
+            return new Apint[] { x, Apint.ZEROS[x.radix()] };
         }
 
         long precision;
@@ -366,7 +371,7 @@ public class ApintMath
 
         if (a.compareTo(b) < 0)
         {
-            return new Apint[] { Apint.ZERO, x };  // abs(x) < abs(y)
+            return new Apint[] { Apint.ZEROS[x.radix()], x };   // abs(x) < abs(y)
         }
         else
         {
@@ -429,7 +434,7 @@ public class ApintMath
     {
         if (a.signum() == 0 && b.signum() == 0)
         {
-            return Apint.ZERO;
+            return Apint.ZEROS[a.radix()];
         }
         else
         {
