@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -30,7 +35,7 @@ import org.apfloat.spi.Util;
  * <code>ApcomplexMath.acos(Apcomplex.ZERO)</code>.
  *
  * @since 1.5
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -298,7 +303,7 @@ public class FixedPrecisionApcomplexHelper
         if (z.real().signum() == 0 && z.imag().signum() == 0)
         {
             // Zero always has infinite precision so when zero input causes nonzero output special care must be taken
-            return valueOf(new Apcomplex(Apfloat.ZERO, pi(z.radix()).divide(new Apfloat(2, precision(), z.radix()))));
+            return valueOf(new Apcomplex(Apfloat.ZEROS[z.radix()], pi(z.radix()).divide(new Apfloat(2, precision(), z.radix()))));
         }
         return valueOf(ApcomplexMath.acosh(setPrecision(z)));
     }
@@ -715,7 +720,7 @@ public class FixedPrecisionApcomplexHelper
     /**
      * Gamma function.
      *
-     * @param z The operand(s).
+     * @param z The operand.
      *
      * @return <code>&Gamma;(z)</code>.
      *
@@ -730,6 +735,61 @@ public class FixedPrecisionApcomplexHelper
         return valueOf(ApcomplexMath.gamma(setGammaPrecision(z)));
     }
 
+    /**
+     * Incomplete gamma function.
+     *
+     * @param a The first operand.
+     * @param z The second operand.
+     *
+     * @return <code>&Gamma;(a, z)</code>.
+     *
+     * @throws ArithmeticException If <code>a</code> is a nonpositive integer and <code>z</code> is zero.
+     *
+     * @since 1.10.0
+     */
+
+    public Apcomplex gamma(Apcomplex a, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.gamma(setGammaPrecision(a), setPrecision(z)));
+    }
+
+    /**
+     * Generalized incomplete gamma function.
+     *
+     * @param a The first operand.
+     * @param z0 The second operand.
+     * @param z1 The third operand.
+     *
+     * @return <code>&Gamma;(a, z0)</code> - <code>&Gamma;(a, z1)</code>.
+     *
+     * @throws ArithmeticException If <code>a</code> is a nonpositive integer and either <code>z0</code> or <code>z1</code> is zero.
+     *
+     * @since 1.10.0
+     */
+
+    public Apcomplex gamma(Apcomplex a, Apcomplex z0, Apcomplex z1)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.gamma(setGammaPrecision(a), setPrecision(z0), setPrecision(z1)));
+    }
+
+    /**
+     * Unit in the last place.
+     *
+     * @param z The operand.
+     *
+     * @return The unit in the last place.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat ulp(Apcomplex z)
+        throws ApfloatRuntimeException
+    {
+        return valueOf(ApcomplexMath.ulp(setPrecision(z)));
+    }
+
     /**
      * Returns the precision, which is used for the results.
      *
