@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -32,7 +37,7 @@ import org.apfloat.spi.Util;
  * <code>ApfloatMath.acos(Apfloat.ZERO)</code>.
  *
  * @since 1.5
- * @version 1.9.0
+ * @version 1.10.0
  * @author Mikko Tommila
  */
 
@@ -876,10 +881,40 @@ public class FixedPrecisionApfloatHelper
         return valueOf(ApfloatMath.sum(setPrecision(x)));
     }
 
+    /**
+     * &gamma;, the Euler-Mascheroni constant.
+     *
+     * @return <code>&gamma;</code>.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat euler()
+        throws ApfloatRuntimeException
+    {
+        return ApfloatMath.euler(precision());
+    }
+
+    /**
+     * &gamma;, the Euler-Mascheroni constant.
+     *
+     * @param radix The radix of the result.
+     *
+     * @return <code>&gamma;</code>.
+     *
+     * @exception java.lang.NumberFormatException If the radix is invalid.
+     */
+
+    public Apfloat euler(int radix)
+        throws NumberFormatException, ApfloatRuntimeException
+    {
+        return ApfloatMath.euler(precision(), radix);
+    }
+
     /**
      * Gamma function.
      *
-     * @param x The operand(s).
+     * @param x The operand.
      *
      * @return <code>&Gamma;(x)</code>.
      *
@@ -894,6 +929,45 @@ public class FixedPrecisionApfloatHelper
         return valueOf(ApfloatMath.gamma(setGammaPrecision(x)));
     }
 
+    /**
+     * Incomplete gamma function.
+     *
+     * @param a The first operand.
+     * @param x The second operand.
+     *
+     * @return <code>&Gamma;(a, x)</code>.
+     *
+     * @throws ArithmeticException If <code>a</code> is not a positive integer and <code>x</code> is nonpositive.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat gamma(Apfloat a, Apfloat x)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.gamma(setGammaPrecision(a), setPrecision(x)));
+    }
+
+    /**
+     * Generalized incomplete gamma function.
+     *
+     * @param a The first operand.
+     * @param x0 The second operand.
+     * @param x1 The third operand.
+     *
+     * @return <code>&Gamma;(a, x0) - &Gamma;(a, x1)</code>.
+     *
+     * @throws ArithmeticException If <code>a</code> is not a positive integer and either <code>x0</code> or <code>x1</code> is nonpositive.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat gamma(Apfloat a, Apfloat x0, Apfloat x1)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.gamma(setGammaPrecision(a), setPrecision(x0), setPrecision(x1)));
+    }
+
     /**
      * Return a uniformly distributed random number <code>0 &le; x &lt; 1</code>.
      *
@@ -965,6 +1039,8 @@ public class FixedPrecisionApfloatHelper
      * @param y The second operand.
      *
      * @return The maximum of <code>x</code> and <code>y</code>.
+     *
+     * @since 1.9.0
      */
 
     public Apfloat max(Apfloat x, Apfloat y)
@@ -980,6 +1056,8 @@ public class FixedPrecisionApfloatHelper
      * @param y The second operand.
      *
      * @return The minimum of <code>x</code> and <code>y</code>.
+     *
+     * @since 1.9.0
      */
 
     public Apfloat min(Apfloat x, Apfloat y)
@@ -988,6 +1066,71 @@ public class FixedPrecisionApfloatHelper
         return valueOf(ApfloatMath.min(setPrecision(x), setPrecision(y)));
     }
 
+    /**
+     * The number adjacent to the first argument in the direction of the second argument.
+     *
+     * @param x The first operand.
+     * @param y The second operand.
+     *
+     * @return The number adjacent to the first argument in the direction of the second argument.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat nextAfter(Apfloat x, Apfloat y)
+        throws ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.nextAfter(setPrecision(x), setPrecision(y)));
+    }
+
+    /**
+     * The adjacent value closer to negative infinity.
+     *
+     * @param x The operand.
+     *
+     * @return The adjacent value closer to negative infinity.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat nextDown(Apfloat x)
+        throws ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.nextDown(setPrecision(x)));
+    }
+
+    /**
+     * The adjacent value closer to positive infinity.
+     *
+     * @param x The operand.
+     *
+     * @return The adjacent value closer to positive infinity.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat nextUp(Apfloat x)
+        throws ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.nextUp(setPrecision(x)));
+    }
+
+    /**
+     * Unit in the last place.
+     *
+     * @param x The operand.
+     *
+     * @return The unit in the last place.
+     *
+     * @since 1.10.0
+     */
+
+    public Apfloat ulp(Apfloat x)
+        throws ApfloatRuntimeException
+    {
+        return valueOf(ApfloatMath.ulp(setPrecision(x)));
+    }
+
     private Apfloat setPrecision(Apfloat x)
         throws ApfloatRuntimeException
     {
