@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -26,7 +31,7 @@ import org.apfloat.spi.Util;
  * Helper class for Lambert W function.
  *
  * @since 1.8.0
- * @version 1.8.1
+ * @version 1.10.0
  * @author Mikko Tommila
  */
 
@@ -381,6 +386,7 @@ class LambertWHelper {
         throws ArithmeticException, ApfloatRuntimeException
     {
         Apcomplex logz = ApcomplexMath.log(z);
+        logz = ApfloatHelper.ensurePrecision(logz, z.precision());  // Issue #10: precision might be very low if z is close to 1, thanks to Axel Kramer for finding this bug
         if (this.k != 0)
         {
             // log(z) + 2*pi*k*i
