@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -33,7 +38,7 @@ import java.util.Locale;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -72,6 +77,7 @@ public class ApcomplexTest
         suite.addTest(new ApcomplexTest("testNumberValuesExact"));
         suite.addTest(new ApcomplexTest("testEqualDigits"));
         suite.addTest(new ApcomplexTest("testEquals"));
+        suite.addTest(new ApcomplexTest("testTest"));
         suite.addTest(new ApcomplexTest("testHashCode"));
         suite.addTest(new ApcomplexTest("testToString"));
         suite.addTest(new ApcomplexTest("testWriteTo"));
@@ -91,6 +97,10 @@ public class ApcomplexTest
         assertEquals("(5, 7) radix", 12, a.radix());
         assertEquals("(5, 7) precision", Apfloat.INFINITE, a.precision());
         assertEquals("(5, 7) String", "(5, 7)", a.toString(true));
+        a = new Apcomplex(new Apfloat(7, Apfloat.DEFAULT, 12));
+        assertEquals("7 imag radix", 12, a.imag().radix());
+        assertEquals("7 precision", Apfloat.INFINITE, a.precision());
+        assertEquals("7 String", "7", a.toString(true));
 
         a = new Apcomplex(new Apfloat(0, Apfloat.DEFAULT, 11), new Apfloat(7, Apfloat.DEFAULT, 12));
         a = new Apcomplex(new Apfloat(7, Apfloat.DEFAULT, 11), new Apfloat(0, Apfloat.DEFAULT, 12));
@@ -123,6 +133,13 @@ public class ApcomplexTest
         a = new Apcomplex("( 5 , 7 )");
         assertEquals("( 5 , 7 ) String", "(5, 7)", a.toString(true));
 
+        ApfloatContext.getContext().setDefaultRadix(12);
+        a = new Apcomplex("(5)");
+        assertEquals("(5) imag radix", 12, a.imag().radix());
+        a = new Apcomplex("5");
+        assertEquals("5 imag radix", 12, a.imag().radix());
+        ApfloatContext.getContext().setDefaultRadix(10);
+
         try
         {
             a = new Apcomplex("(3,4");
@@ -157,6 +174,15 @@ public class ApcomplexTest
         a = new Apcomplex(in);
         assertEquals("( 5 , 7 ) String", "(5, 7)", a.toString(true));
 
+        ApfloatContext.getContext().setDefaultRadix(12);
+        in = new PushbackReader(new StringReader("(5)"));
+        a = new Apcomplex(in);
+        assertEquals("(5) imag radix", 12, a.imag().radix());
+        in = new PushbackReader(new StringReader("5"));
+        a = new Apcomplex(in);
+        assertEquals("5 imag radix", 12, a.imag().radix());
+        ApfloatContext.getContext().setDefaultRadix(10);
+
         try
         {
             a = new Apcomplex(new PushbackReader(new StringReader("(3,4")));
@@ -351,6 +377,13 @@ public class ApcomplexTest
 
         a = new Apcomplex(new Apfloat(0), new Apfloat(0));
         assertEquals("(0, 0) -> 6 value", new Apcomplex(new Apfloat(0), new Apfloat(0)), a.precision(6));
+
+        a = new Apcomplex(new Apfloat("1", 1, 12), new Apfloat("10", 2, 12));
+        assertEquals("(1, 10) -> 1 value", new Apcomplex(Apfloat.ZERO, new Apfloat("10", 1, 12)), a.precision(1));
+        assertEquals("(1, 10) -> 1 real radix", 12, a.precision(1).real().radix());
+        a = new Apcomplex(new Apfloat("10", 2, 12), new Apfloat("1", 1, 12));
+        assertEquals("(10, 1) -> 1 value", new Apcomplex(new Apfloat("10", 1, 12), Apfloat.ZERO), a.precision(1));
+        assertEquals("(10, 1) -> 1 imag radix", 12, a.precision(1).imag().radix());
     }
 
     public static void testNegate()
@@ -648,6 +681,24 @@ public class ApcomplexTest
         a = new Apcomplex("(5,6)");
         b = new Apcomplex("(5,5)");
         assertEquals("(5,6) == (5,5)", false, a.equals(b));
+        a = new Apcomplex("(5,5)");
+        b = new Apcomplex("(6,5)");
+        assertEquals("(5,5) == (6,5)", false, a.equals(b));
+        a = new Apcomplex("(6,5)");
+        b = new Apcomplex("(5,5)");
+        assertEquals("(6,5) == (5,5)", false, a.equals(b));
+        a = new Apcomplex("(5,-5)");
+        b = new Apcomplex("(5,5)");
+        assertEquals("(5,-5) == (5,5)", false, a.equals(b));
+        a = new Apcomplex("(5,5)");
+        b = new Apcomplex("(5,-5)");
+        assertEquals("(5,5) == (5,-5)", false, a.equals(b));
+        a = new Apcomplex("(-5,5)");
+        b = new Apcomplex("(5,5)");
+        assertEquals("(-5,5) == (5,5)", false, a.equals(b));
+        a = new Apcomplex("(5,5)");
+        b = new Apcomplex("(-5,5)");
+        assertEquals("(5,5) == (-5,5)", false, a.equals(b));
         a = new Apcomplex("(6,6)");
         b = new Apcomplex("(6,6)");
         assertEquals("(6,6) == (6,6)", true, a.equals(b));
@@ -656,6 +707,19 @@ public class ApcomplexTest
         assertEquals("(6,6) == something else", false, a.equals("bogus"));
     }
 
+    public static void testTest()
+    {
+        Apcomplex a = new Apcomplex("0"),
+                  b = new Apcomplex("0");
+        assertTrue("0 test 0", a.test(b));
+        a = new Apcomplex("0");
+        b = new Apcomplex("1");
+        assertFalse("0 test 1", a.test(b));
+        a = new Apcomplex("(0,0)");
+        b = new Apcomplex("(0,1)");
+        assertFalse("0 test i", a.test(b));
+    }
+
     public static void testHashCode()
     {
         Apcomplex a = new Apcomplex("0"),
