@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -24,7 +29,7 @@ import java.util.Map;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -75,6 +80,8 @@ public class ApintMathTest
         assertEquals("2^-3", new Apint(0), ApintMath.pow(x, -3));
         assertEquals("2^0", new Apint(1), ApintMath.pow(x, 0));
 
+        assertEquals("2^-3 radix", 12, ApintMath.pow(new Apint(2, 12), -3).radix());
+
         try
         {
             ApintMath.pow(new Apint(0), 0);
@@ -179,6 +186,9 @@ public class ApintMathTest
         assertEquals("-1st root quot", new Apint(0), ApintMath.root(new Apint(2), -1)[0]);
         assertEquals("-1st root rem", new Apint(2), ApintMath.root(new Apint(2), -1)[1]);
 
+        assertEquals("1st root rem radix", 12, ApintMath.root(new Apint(2, 12), 1)[1].radix());
+        assertEquals("-1st root quot radix", 12, ApintMath.root(new Apint(2, 12), -1)[0].radix());
+
         try
         {
             ApintMath.root(new Apint(-2), 2);
@@ -316,6 +326,11 @@ public class ApintMathTest
         assertEquals("-very long / -very long", new Apint("88887588172852973806344237990766854843278434484762916030592876582400800681711796943572298887271879881107698627193143861585371238964537515138970410491320347"), r[0]);
         assertEquals("-very long % -very long", new Apint("-6864797660130609714981900799081393217269435300143305409394463459185543183397656052122559640661454554977296311391480858037121987999716643812574028291115057150"), r[1]);
 
+        r = ApintMath.div(new Apint(1, 12), new Apint(3, 12));
+        assertEquals("1 / 3 radix", 12, r[0].radix());
+        r = ApintMath.div(new Apint(3, 12), new Apint(1, 12));
+        assertEquals("3 % 1 radix", 12, r[1].radix());
+
         try
         {
             ApintMath.div(new Apint(3), new Apint(0));
@@ -336,6 +351,8 @@ public class ApintMathTest
         assertEquals("3 gcd 1", new Apint(1), ApintMath.gcd(new Apint(3), new Apint(1)));
         assertEquals("0 gcd 3", new Apint(3), ApintMath.gcd(new Apint(0), new Apint(3)));
         assertEquals("3 gcd 0", new Apint(3), ApintMath.gcd(new Apint(3), new Apint(0)));
+        assertEquals("0 gcd -3", new Apint(3), ApintMath.gcd(new Apint(0), new Apint(-3)));
+        assertEquals("-3 gcd 0", new Apint(3), ApintMath.gcd(new Apint(-3), new Apint(0)));
         assertEquals("0 gcd 0", new Apint(0), ApintMath.gcd(new Apint(0), new Apint(0)));
         Apint a = new Apfloat("1e100000").truncate().add(new Apint(1));
         assertEquals("1e100000+1 gcd 0", a, ApintMath.gcd(a, new Apint(0)));
@@ -352,6 +369,7 @@ public class ApintMathTest
         assertEquals("0 lcm 3", new Apint(0), ApintMath.lcm(new Apint(0), new Apint(3)));
         assertEquals("3 lcm 0", new Apint(0), ApintMath.lcm(new Apint(3), new Apint(0)));
         assertEquals("0 lcm 0", new Apint(0), ApintMath.lcm(new Apint(0), new Apint(0)));
+        assertEquals("0 lcm 0 radix", 12, ApintMath.lcm(new Apint(0, 12), new Apint(0, 12)).radix());
     }
 
     public static void testModMultiply()
@@ -537,7 +555,7 @@ public class ApintMathTest
         for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
         {
             Map<Apint, Integer> counts = new HashMap<>();
-            for (int i = 0; i < 10 * radix; i++)
+            for (int i = 0; i < 20 * radix; i++)
             {
                 Apint a = ApintMath.random(1, radix);
                 counts.put(a, counts.getOrDefault(a, 0) + 1);
