@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -33,7 +38,7 @@ import java.util.IllegalFormatException;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -82,6 +87,7 @@ public class ApintTest
         suite.addTest(new ApintTest("testToBigInteger"));
         suite.addTest(new ApintTest("testCompareTo"));
         suite.addTest(new ApintTest("testEquals"));
+        suite.addTest(new ApintTest("testTest"));
         suite.addTest(new ApintTest("testHashCode"));
         suite.addTest(new ApintTest("testToString"));
         suite.addTest(new ApintTest("testWriteTo"));
@@ -301,6 +307,7 @@ public class ApintTest
         assertEquals("0 / 4", new Apint(0), new Apint(0).divide(a));
         assertEquals("4 / 1", new Apint(4, 12), a.divide(new Apint(1, 12)));
         assertEquals("4 / ONE", new Apint(4, 12), a.divide(Apint.ONE));
+        assertEquals("4 / 3 radix", 12, b.divide(a).radix());
 
         try
         {
@@ -356,10 +363,11 @@ public class ApintTest
 
     public static void testFrac()
     {
-        Apint a = new Apint("2");
+        Apint a = new Apint("2", 12);
         assertEquals("2 frac", new Apint(0), a.frac());
+        assertEquals("2 frac radix", 12, a.frac().radix());
         a = new Apint(0);
-        assertEquals("0 truncate", new Apint(0), a.frac());
+        assertEquals("0 frac", new Apint(0), a.frac());
     }
 
     public static void testRoundAway()
@@ -546,6 +554,44 @@ public class ApintTest
         assertEquals("obj float 3 == 3", true, obj2.equals(obj1));
     }
 
+    public static void testTest()
+    {
+        Apint a = new Apint(0),
+              b = new Apint(0);
+        assertTrue("0 test 0", a.test(b));
+        a = new Apint(0);
+        b = new Apint(1);
+        assertFalse("0 test 1", a.test(b));
+        a = new Apint(-1);
+        b = new Apint(1);
+        assertFalse("-1 test 1", a.test(b));
+
+        a = new Apint(2);
+        Aprational r = new Aprational("2");
+        assertTrue("2 test 2, type", a.test(r));
+        assertTrue("2 test 2, type reverse", r.test(a));
+
+        a = new Apint(2);
+        r = new Aprational("2/3");
+        assertTrue("2 test 2/3, type", a.test(r));
+        assertTrue("2/3 test 2, type", r.test(a));
+
+        a = new Apint(2);
+        r = new Aprational("2/13");
+        assertFalse("2 test 2/13, type", a.test(r));
+        assertFalse("2/13 test 2, type", r.test(a));
+
+        a = new Apint(2);
+        Apfloat f = new Apfloat(2);
+        assertTrue("2 test 2, type", a.test(f));
+        assertTrue("2 test 2, type reverse", f.test(a));
+
+        a = new Apint(2);
+        f = new Apfloat(0.6);
+        assertFalse("2 test 0.6, type", a.test(f));
+        assertFalse("0.6 test 2, type", f.test(a));
+    }
+
     public static void testHashCode()
     {
         Apint a = new Apint(0),
