@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -33,7 +38,7 @@ import java.util.IllegalFormatException;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.0
+ * @version 1.10.0
  * @author Mikko Tommila
  */
 
@@ -82,6 +87,7 @@ public class AprationalTest
         suite.addTest(new AprationalTest("testEqualDigits"));
         suite.addTest(new AprationalTest("testCompareTo"));
         suite.addTest(new AprationalTest("testEquals"));
+        suite.addTest(new AprationalTest("testTest"));
         suite.addTest(new AprationalTest("testHashCode"));
         suite.addTest(new AprationalTest("testToString"));
         suite.addTest(new AprationalTest("testWriteTo"));
@@ -768,6 +774,38 @@ public class AprationalTest
         assertEquals("obj 0.3333333333 == 1/3", false, obj2.equals(obj1));
     }
 
+    public static void testTest()
+    {
+        Aprational a = new Aprational("0"),
+                   b = new Aprational("0");
+        assertTrue("0 test 0", a.test(b));
+        a = new Aprational("0");
+        b = new Aprational("1");
+        assertFalse("0 test 1", a.test(b));
+        a = new Aprational("-1");
+        b = new Aprational("1");
+        assertFalse("-1 test 1", a.test(b));
+        a = new Aprational("1");
+        b = new Aprational("1/10");
+        assertFalse("1 test 1/10", a.test(b));
+        a = new Aprational("2", 10);
+        b = new Aprational("2", 11);
+        assertTrue("2 test 2, radix", a.test(b));
+
+        a = new Aprational("2");
+        Apfloat f = new Apfloat(2);
+        assertTrue("2 test 2, type", a.test(f));
+        assertTrue("2 test 2, type reverse", f.test(a));
+        a = new Aprational("2/3");
+        f = new Apfloat(0.07);
+        assertTrue("2/3 test 0.07, type", a.test(f));
+        assertTrue("0.07 test 2/3, type", f.test(a));
+        a = new Aprational("3");
+        f = new Apfloat(0.07);
+        assertFalse("3 test 0.07, type", a.test(f));
+        assertFalse("0.07 test 3, type", f.test(a));
+    }
+
     public static void testHashCode()
     {
         Aprational a = new Aprational("1/2"),
