@@ -1,27 +1,32 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.0
+ * @version 1.10.1
  * @author Mikko Tommila
  */
 
@@ -82,6 +87,9 @@ public class FixedPrecisionApcomplexHelperTest
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testProduct"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testSum"));
         suite.addTest(new FixedPrecisionApcomplexHelperTest("testGamma"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testGammaIncomplete"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testGammaIncompleteGeneralized"));
+        suite.addTest(new FixedPrecisionApcomplexHelperTest("testUlp"));
 
         return suite;
     }
@@ -390,8 +398,12 @@ public class FixedPrecisionApcomplexHelperTest
 
         z = Apcomplex.ZERO;
         result = helper.acosh(z);
-        assertEquals("value", new Apcomplex(Apfloat.ZERO, new Apfloat("1.570796326794896619231321691639751442098584699687552910487472296153908203143104499314017412671058534")), result, new Apfloat("1e-98"));
-        assertEquals("precision", 100, result.precision());
+        assertEquals("zero value", new Apcomplex(Apfloat.ZERO, new Apfloat("1.570796326794896619231321691639751442098584699687552910487472296153908203143104499314017412671058534")), result, new Apfloat("1e-98"));
+        assertEquals("zero precision", 100, result.precision());
+
+        z = new Apint(0, 12);
+        result = helper.acosh(z);
+        assertEquals("zero real radix", 12, result.real().radix());
     }
 
     public static void testAsinh()
@@ -574,4 +586,33 @@ public class FixedPrecisionApcomplexHelperTest
         assertEquals("value", new Apcomplex("(2.2799397381057012808806415e-682188176916,2.56143734228029034694359025e-682188176915)"), result, new Apfloat("5e-682188176941"));
         assertEquals("precision", 26, result.precision());
     }
+
+    public static void testGammaIncomplete()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(5);
+        Apcomplex a = new Apcomplex("(1,2)");
+        Apcomplex z = new Apcomplex("(3,4)");
+        Apcomplex result = helper.gamma(a, z);
+        assertEquals("value", new Apcomplex("(0.0087875,-0.0046329)"), result, new Apfloat("5e-7"));
+        assertEquals("precision", 5, result.precision());
+    }
+
+    public static void testGammaIncompleteGeneralized()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(6);
+        Apcomplex a = new Apcomplex("(1,2)");
+        Apcomplex z0 = new Apcomplex("(3,4)");
+        Apcomplex z1 = new Apcomplex("(5,6)");
+        Apcomplex result = helper.gamma(a, z0, z1);
+        assertEquals("value", new Apcomplex("(0.00896428,-0.00326815)"), result, new Apfloat("5e-8"));
+        assertEquals("precision", 6, result.precision());
+    }
+
+    public static void testUlp()
+    {
+        FixedPrecisionApcomplexHelper helper = new FixedPrecisionApcomplexHelper(5);
+        Apfloat result = helper.ulp(new Apcomplex("(10,2)"));
+        assertEquals("(10,2) value", new Apfloat("0.001"), result);
+        assertEquals("(10,2) precision", 5, result.precision());
+    }
 }
