@@ -1,20 +1,25 @@
 /*
- * Apfloat arbitrary precision arithmetic library
- * Copyright (C) 2002-2019  Mikko Tommila
+ * MIT License
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * Copyright (c) 2002-2021 Mikko Tommila
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 package org.apfloat;
 
@@ -23,7 +28,7 @@ import java.math.RoundingMode;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.9.0
+ * @version 1.10.0
  * @author Mikko Tommila
  */
 
@@ -94,11 +99,18 @@ public class FixedPrecisionApfloatHelperTest
         suite.addTest(new FixedPrecisionApfloatHelperTest("testToRadians"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testProduct"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testSum"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testEuler"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testGamma"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testGammaIncomplete"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testGammaIncompleteGeneralized"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testRandom"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testRandomGaussian"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testMax"));
         suite.addTest(new FixedPrecisionApfloatHelperTest("testMin"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testNextAfter"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testNextDown"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testNextUp"));
+        suite.addTest(new FixedPrecisionApfloatHelperTest("testUlp"));
 
         return suite;
     }
@@ -877,6 +889,28 @@ public class FixedPrecisionApfloatHelperTest
         assertEquals("precision", 10, result.precision());
     }
 
+    public static void testEuler()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(100);
+        Apfloat result = helper.euler();
+        assertEquals("value", new Apfloat("0.5772156649015328606065120900824024310421593359399235988057672348848677267776646709369470632917467495"), result, new Apfloat("5e-100"));
+        assertEquals("precision", 100, result.precision());
+
+        result = helper.euler(16);
+        assertEquals("value 16", new Apfloat("0.93c467e37db0c7a4d1be3f810152cb56a1cecc3af65cc0190c03df34709affbd8e4b59fa03a9f0eed0649ccb621057d11057", Apfloat.DEFAULT, 16), result, new Apfloat(1e-100, 1, 16));
+        assertEquals("precision 16", 100, result.precision());
+
+        try
+        {
+            helper.euler(1);
+            fail("Radix 1 accepted");
+        }
+        catch (NumberFormatException nfe)
+        {
+            // OK; invalid radix
+        }
+    }
+
     public static void testGamma()
     {
         FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(27);
@@ -886,6 +920,27 @@ public class FixedPrecisionApfloatHelperTest
         assertEquals("precision", 27, result.precision());
     }
 
+    public static void testGammaIncomplete()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(5);
+        Apfloat a = new Apfloat("1.5");
+        Apfloat x = new Apfloat("3.5");
+        Apfloat result = helper.gamma(a, x);
+        assertEquals("value", new Apfloat("0.063718"), result, new Apfloat("5e-6"));
+        assertEquals("precision", 5, result.precision());
+    }
+
+    public static void testGammaIncompleteGeneralized()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(6);
+        Apfloat a = new Apfloat("1.5");
+        Apfloat x0 = new Apfloat("2.5");
+        Apfloat x1 = new Apfloat("3.5");
+        Apfloat result = helper.gamma(a, x0, x1);
+        assertEquals("value", new Apfloat("0.0885335"), result, new Apfloat("5e-7"));
+        assertEquals("precision", 6, result.precision());
+    }
+
     public static void testRandom()
     {
         FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(100);
@@ -960,4 +1015,36 @@ public class FixedPrecisionApfloatHelperTest
         assertEquals("2,1 value", new Apfloat("1"), result);
         assertEquals("2,1 precision", 100, result.precision());
     }
+
+    public static void testNextAfter()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(5);
+        Apfloat result = helper.nextAfter(new Apfloat(1), new Apfloat(2));
+        assertEquals("1,2 value", new Apfloat("1.0001"), result);
+        assertEquals("1,2 precision", 5, result.precision());
+    }
+
+    public static void testNextDown()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(5);
+        Apfloat result = helper.nextDown(new Apfloat(1));
+        assertEquals("1 value", new Apfloat("0.9999"), result);
+        assertEquals("1 precision", 5, result.precision());
+    }
+
+    public static void testNextUp()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(5);
+        Apfloat result = helper.nextUp(new Apfloat(1));
+        assertEquals("1 value", new Apfloat("1.0001"), result);
+        assertEquals("1 precision", 5, result.precision());
+    }
+
+    public static void testUlp()
+    {
+        FixedPrecisionApfloatHelper helper = new FixedPrecisionApfloatHelper(5);
+        Apfloat result = helper.ulp(new Apfloat(1));
+        assertEquals("1 value", new Apfloat("0.0001"), result);
+        assertEquals("1 precision", 5, result.precision());
+    }
 }
